@@ -30,6 +30,7 @@
 #include "nm-device-wifi.h"
 #include "nm-device-modem.h"
 #include "nm-device-bt.h"
+#include "nm-device-olpc-mesh.h"
 #include "nm-device-wimax.h"
 #include "nm-device-infiniband.h"
 #include "nm-device.h"
@@ -160,10 +161,9 @@ register_properties (NMDevice *device)
 		{ NM_DEVICE_STATE_REASON,      &priv->state, demarshal_state_reason },
 		{ NM_DEVICE_ACTIVE_CONNECTION, &priv->active_connection, NULL, NM_TYPE_ACTIVE_CONNECTION },
 
-		/* The D-Bus interface has this property, but we don't; register
-		 * it so that handle_property_changed() doesn't complain.
-		 */
+		/* Properties that exist in D-Bus but that we don't track */
 		{ "ip4-address", NULL },
+		{ "device-type", NULL },
 
 		{ NULL },
 	};
@@ -182,8 +182,16 @@ device_state_changed (DBusGProxy *proxy,
 {
 	NMDevice *self = NM_DEVICE (user_data);
 
-	if (old_state != new_state)
+	if (old_state != new_state) {
+		/* Update state here since the PropertyChanged signal for state
+		 * might come in a bit later, but a client might ask for the
+		 * state via nm_device_get_state() as a result of this signal.
+		 * When the PC signal does come in that will trigger the glib
+		 * property notify signal so we don't need to do that here.
+		 */
+		NM_DEVICE_GET_PRIVATE (self)->state = new_state;
 		g_signal_emit (self, signals[STATE_CHANGED], 0, new_state, old_state, reason);
+	}
 }
 
 static void
@@ -629,6 +637,8 @@ _nm_device_gtype_from_dtype (NMDeviceType dtype)
 		return NM_TYPE_DEVICE_MODEM;
 	case NM_DEVICE_TYPE_BT:
 		return NM_TYPE_DEVICE_BT;
+	case NM_DEVICE_TYPE_OLPC_MESH:
+		return NM_TYPE_DEVICE_OLPC_MESH;
 	case NM_DEVICE_TYPE_WIMAX:
 		return NM_TYPE_DEVICE_WIMAX;
 	case NM_DEVICE_TYPE_INFINIBAND:
@@ -823,6 +833,8 @@ nm_device_get_device_type (NMDevice *self)
 			priv->device_type = NM_DEVICE_TYPE_MODEM;
 		else if (NM_IS_DEVICE_BT (self))
 			priv->device_type = NM_DEVICE_TYPE_BT;
+		else if (NM_IS_DEVICE_OLPC_MESH (self))
+			priv->device_type = NM_DEVICE_TYPE_OLPC_MESH;
 		else if (NM_IS_DEVICE_WIMAX (self))
 			priv->device_type = NM_DEVICE_TYPE_WIMAX;
 		else if (NM_IS_DEVICE_INFINIBAND (self))
