@@ -45,12 +45,15 @@ enum {
 	LOGD_VPN        = 0x00004000,
 	LOGD_SHARING    = 0x00008000, /* Connection sharing/dnsmasq */
 	LOGD_SUPPLICANT = 0x00010000, /* WiFi and 802.1x */
-	LOGD_USER_SET   = 0x00020000, /* User settings */
-	LOGD_SYS_SET    = 0x00040000, /* System settings */
+	LOGD_AGENTS     = 0x00020000, /* Secret agents */
+	LOGD_SETTINGS   = 0x00040000, /* Settings */
 	LOGD_SUSPEND    = 0x00080000, /* Suspend/Resume */
 	LOGD_CORE       = 0x00100000, /* Core daemon and policy stuff */
 	LOGD_DEVICE     = 0x00200000, /* Device state and activation */
 	LOGD_OLPC_MESH  = 0x00400000,
+	LOGD_WIMAX      = 0x00800000,
+	LOGD_INFINIBAND = 0x01000000,
+	LOGD_FIREWALL   = 0x02000000,
 };
 
 #define LOGD_DHCP (LOGD_DHCP4 | LOGD_DHCP6)
@@ -63,10 +66,13 @@ enum {
 	LOGL_DEBUG = 0x00000008
 };
 
+typedef enum {
+    NM_LOGGING_ERROR_UNKNOWN_LEVEL = 0,
+    NM_LOGGING_ERROR_UNKNOWN_DOMAIN = 1,
+} NMLoggingError;
+
 #define NM_LOGGING_ERROR (nm_logging_error_quark ())
-#define NM_TYPE_LOGGING_ERROR (nm_logging_error_get_type ())
 GQuark nm_logging_error_quark    (void);
-GType  nm_logging_error_get_type (void);
 
 
 #define nm_log_err(domain, ...) \
@@ -94,6 +100,7 @@ void _nm_log (const char *loc,
 const char *nm_logging_level_to_string (void);
 char *nm_logging_domains_to_string (void);
 gboolean nm_logging_level_enabled (guint32 level);
+gboolean nm_logging_domain_enabled (guint32 domain);
 
 /* Undefine the nm-utils.h logging stuff to ensure errors */
 #undef nm_print_backtrace
