@@ -38,6 +38,9 @@ NMConfig *nm_config_new (const char *cli_config_path,
                          const char *cli_plugins,
                          const char *cli_log_level,
                          const char *cli_log_domains,
+                         const char *cli_connectivity_check_uri,
+                         const gint connectivity_check_interval,
+                         const char *cli_connectivity_check_response,
                          GError **error);
 
 const char *nm_config_get_path (NMConfig *config);
@@ -46,6 +49,9 @@ const char *nm_config_get_dhcp_client (NMConfig *config);
 const char **nm_config_get_dns_plugins (NMConfig *config);
 const char *nm_config_get_log_level (NMConfig *config);
 const char *nm_config_get_log_domains (NMConfig *config);
+const char *nm_config_get_connectivity_uri (NMConfig *config);
+const guint nm_config_get_connectivity_interval (NMConfig *config);
+const char *nm_config_get_connectivity_response (NMConfig *config);
 
 void nm_config_free (NMConfig *config);
 
