@@ -375,7 +375,11 @@ real_get_generic_capabilities (NMDevice *dev)
 }
 
 static gboolean
-hwaddr_matches (NMDevice *device, NMConnection *connection, gboolean fail_if_no_hwaddr)
+hwaddr_matches (NMDevice *device,
+                NMConnection *connection,
+                const guint8 *other_hwaddr,
+                guint other_hwaddr_len,
+                gboolean fail_if_no_hwaddr)
 {
 	NMDeviceBtPrivate *priv = NM_DEVICE_BT_GET_PRIVATE (device);
 	NMSettingBluetooth *s_bt;
@@ -392,7 +396,12 @@ hwaddr_matches (NMDevice *device, NMConnection *connection, gboolean fail_if_no_
 		g_return_val_if_fail (devmac != NULL, FALSE);
 		g_return_val_if_fail (devmac->len == mac->len, FALSE);
 
-		matches = (memcmp (mac->data, devmac->data, mac->len) == 0) ? TRUE : FALSE;
+		if (other_hwaddr) {
+			g_return_val_if_fail (other_hwaddr_len == devmac->len, FALSE);
+			matches = (memcmp (mac->data, other_hwaddr, mac->len) == 0) ? TRUE : FALSE;
+		} else
+			matches = (memcmp (mac->data, devmac->data, mac->len) == 0) ? TRUE : FALSE;
+
 		g_byte_array_free (devmac, TRUE);
 		return matches;
 	} else if (fail_if_no_hwaddr == FALSE)
@@ -845,8 +854,11 @@ real_act_stage2_config (NMDevice *device, NMDeviceStateReason *reason)
 	NMDBusManager *dbus_mgr;
 	DBusGConnection *g_connection;
 	gboolean dun = FALSE;
+	NMConnection *connection;
 
-	priv->bt_type = get_connection_bt_type (nm_device_get_connection (device));
+	connection = nm_device_get_connection (device);
+	g_assert (connection);
+	priv->bt_type = get_connection_bt_type (connection);
 	if (priv->bt_type == NM_BT_CAPABILITY_NONE) {
 		// FIXME: set a reason code
 		return NM_ACT_STAGE_RETURN_FAILURE;
