@@ -241,6 +241,29 @@ carrier_off (NMNetlinkMonitor *monitor,
 	}
 }
 
+static gboolean
+get_carrier_sync (NMDeviceWired *self)
+{
+	NMDeviceWiredPrivate *priv = NM_DEVICE_WIRED_GET_PRIVATE (self);
+	GError *error = NULL;
+	guint32 ifflags = 0;
+
+	/* Get initial link state */
+	if (!nm_netlink_monitor_get_flags_sync (priv->monitor,
+	                                        nm_device_get_ip_ifindex (NM_DEVICE (self)),
+	                                        &ifflags,
+	                                        &error)) {
+		nm_log_warn (LOGD_HW | NM_DEVICE_WIRED_LOG_LEVEL (NM_DEVICE (self)),
+		             "(%s): couldn't get carrier state: (%d) %s",
+		             nm_device_get_ip_iface (NM_DEVICE (self)),
+		             error ? error->code : -1,
+		             (error && error->message) ? error->message : "unknown");
+		g_clear_error (&error);
+	}
+
+	return !!(ifflags & IFF_LOWER_UP);
+}
+
 static GObject*
 constructor (GType type,
 			 guint n_construct_params,
@@ -280,9 +303,6 @@ constructor (GType type,
 
 	caps = nm_device_get_capabilities (self);
 	if (caps & NM_DEVICE_CAP_CARRIER_DETECT) {
-		GError *error = NULL;
-		guint32 ifflags = 0;
-
 		/* Only listen to netlink for cards that support carrier detect */
 		priv->monitor = nm_netlink_monitor_get ();
 
@@ -293,19 +313,7 @@ constructor (GType type,
 		                                               G_CALLBACK (carrier_off),
 		                                               self);
 
-		/* Get initial link state */
-		if (!nm_netlink_monitor_get_flags_sync (priv->monitor,
-		                                        nm_device_get_ifindex (NM_DEVICE (self)),
-		                                        &ifflags,
-		                                        &error)) {
-			nm_log_warn (LOGD_HW | NM_DEVICE_WIRED_LOG_LEVEL (NM_DEVICE (self)),
-			             "(%s): couldn't get initial carrier state: (%d) %s",
-			             nm_device_get_iface (NM_DEVICE (self)),
-			             error ? error->code : -1,
-			             (error && error->message) ? error->message : "unknown");
-			g_clear_error (&error);
-		} else
-			priv->carrier = !!(ifflags & IFF_LOWER_UP);
+		priv->carrier = get_carrier_sync (NM_DEVICE_WIRED (self));
 
 		nm_log_info (LOGD_HW | NM_DEVICE_WIRED_LOG_LEVEL (NM_DEVICE (self)),
 		             "(%s): carrier is %s",
@@ -341,7 +349,14 @@ real_hw_is_up (NMDevice *device)
 static gboolean
 real_hw_bring_up (NMDevice *dev, gboolean *no_firmware)
 {
-	return nm_system_iface_set_up (nm_device_get_ip_ifindex (dev), TRUE, no_firmware);
+	gboolean success, carrier;
+
+	success = nm_system_iface_set_up (nm_device_get_ip_ifindex (dev), TRUE, no_firmware);
+	if (success) {
+		carrier = get_carrier_sync (NM_DEVICE_WIRED (dev));
+		set_carrier (NM_DEVICE_WIRED (dev), carrier, carrier ? FALSE : TRUE);
+	}
+	return success;
 }
 
 static void
@@ -416,85 +431,6 @@ real_is_available (NMDevice *dev)
 	return TRUE;
 }
 
-static gboolean
-ip4_match_config (NMDevice *self, NMConnection *connection)
-{
-	NMSettingIP4Config *s_ip4;
-	int i, num;
-	GSList *leases, *iter;
-	NMDHCPManager *dhcp_mgr;
-	const char *method;
-
-	s_ip4 = nm_connection_get_setting_ip4_config (connection);
-	if (!s_ip4)
-		return FALSE;
-
-	/* Get any saved leases that apply to this connection */
-	dhcp_mgr = nm_dhcp_manager_get ();
-	leases = nm_dhcp_manager_get_lease_config (dhcp_mgr,
-	                                           nm_device_get_iface (self),
-	                                           nm_connection_get_uuid (connection));
-	g_object_unref (dhcp_mgr);
-
-	method = nm_setting_ip4_config_get_method (s_ip4);
-	if (!strcmp (method, NM_SETTING_IP4_CONFIG_METHOD_AUTO)) {
-		gboolean found = FALSE;
-
-		/* Find at least one lease's address on the device */
-		for (iter = leases; iter; iter = g_slist_next (iter)) {
-			NMIP4Config *ip4_config = iter->data;
-			NMIP4Address *addr = nm_ip4_config_get_address (ip4_config, 0);
-			struct in_addr tmp = { .s_addr = nm_ip4_address_get_address (addr) };
-
-			if (addr && nm_netlink_find_address (nm_device_get_ip_ifindex (self),
-			                                     AF_INET,
-			                                     &tmp,
-			                                     nm_ip4_address_get_prefix (addr))) {
-				found = TRUE; /* Yay, device has same address as a lease */
-				break;
-			}
-		}
-		g_slist_foreach (leases, (GFunc) g_object_unref, NULL);
-		g_slist_free (leases);
-		return found;
-	} else {
-		/* Maybe the connection used to be DHCP and there are stale leases; ignore them */
-		g_slist_foreach (leases, (GFunc) g_object_unref, NULL);
-		g_slist_free (leases);
-	}
-
-	if (!strcmp (method, NM_SETTING_IP4_CONFIG_METHOD_DISABLED)) {
-		// FIXME: Enforce no ipv4 addresses?
-		return TRUE;
-	}
-
-	/* 'shared' and 'link-local' aren't supported methods because 'shared'
-	 * requires too much iptables and dnsmasq state to be reclaimed, and
-	 * avahi-autoipd isn't smart enough to allow the link-local address to be
-	 * determined at any point other than when it was first assigned.
-	 */
-	if (strcmp (method, NM_SETTING_IP4_CONFIG_METHOD_MANUAL))
-		return FALSE;
-
-	/* Everything below for static addressing */
-
-	/* Find all IP4 addresses of this connection on the device */
-	num = nm_setting_ip4_config_get_num_addresses (s_ip4);
-	for (i = 0; i < num; i++) {
-		NMIP4Address *addr = nm_setting_ip4_config_get_address (s_ip4, i);
-		struct in_addr tmp = { .s_addr = nm_ip4_address_get_address (addr) };
-
-		if (!nm_netlink_find_address (nm_device_get_ip_ifindex (self),
-		                              AF_INET,
-		                              &tmp,
-		                              nm_ip4_address_get_prefix (addr)))
-			return FALSE;
-	}
-
-	/* Success; all the connection's static IP addresses are assigned to the device */
-	return TRUE;
-}
-
 static NMConnection *
 connection_match_config (NMDevice *self, const GSList *connections)
 {
@@ -503,7 +439,7 @@ connection_match_config (NMDevice *self, const GSList *connections)
 	for (iter = connections; iter; iter = iter->next) {
 		NMConnection *candidate = NM_CONNECTION (iter->data);
 
-		if (!ip4_match_config (self, candidate))
+		if (!nm_device_match_ip_config (self, candidate))
 			continue;
 
 		return candidate;
@@ -564,9 +500,9 @@ nm_device_wired_class_init (NMDeviceWiredClass *klass)
  *
  * Get a device's hardware address
  *
- * Return value: (transfer none): @dev's hardware address
+ * Returns: (transfer none): @dev's hardware address
  */
-guint8 *
+const guint8 *
 nm_device_wired_get_hwaddr (NMDeviceWired *dev)
 {
 	NMDeviceWiredPrivate *priv;
@@ -577,13 +513,37 @@ nm_device_wired_get_hwaddr (NMDeviceWired *dev)
 	return priv->hw_addr;
 }
 
+/**
+ * nm_device_wired_set_hwaddr:
+ * @dev: an #NMDeviceWired
+ * @addr: the new hardware address, @addrlen bytes in length
+ * @addrlen: the length in bytes of @addr
+ *
+ * Sets the device's hardware address.
+ */
+void
+nm_device_wired_set_hwaddr (NMDeviceWired *dev,
+                            const guint8 *addr,
+                            guint addrlen)
+{
+	NMDeviceWiredPrivate *priv;
+
+	g_return_if_fail (dev != NULL);
+	g_return_if_fail (addr != NULL);
+
+	priv = NM_DEVICE_WIRED_GET_PRIVATE (dev);
+	g_return_if_fail (addrlen == priv->hw_addr_len);
+
+	memcpy (priv->hw_addr, addr, priv->hw_addr_len);
+}
+
 /**
  * nm_device_wired_get_hwaddr_type:
  * @dev: an #NMDeviceWired
  *
  * Get the type of a device's hardware address
  *
- * Return value: the type of @dev's hardware address
+ * Returns: the type of @dev's hardware address
  */
 int
 nm_device_wired_get_hwaddr_type (NMDeviceWired *dev)
