@@ -45,6 +45,7 @@ typedef enum {
 	NM_MANAGER_ERROR_ALREADY_ASLEEP_OR_AWAKE,
 	NM_MANAGER_ERROR_ALREADY_ENABLED_OR_DISABLED,
 	NM_MANAGER_ERROR_UNSUPPORTED_CONNECTION_TYPE,
+	NM_MANAGER_ERROR_DEPENDENCY_FAILED,
 } NMManagerError;
 
 #define NM_MANAGER_VERSION "version"
@@ -84,7 +85,10 @@ NMManager *nm_manager_new (NMSettings *settings,
                            gboolean initial_net_enabled,
                            gboolean initial_wifi_enabled,
                            gboolean initial_wwan_enabled,
-						   gboolean initial_wimax_enabled,
+                           gboolean initial_wimax_enabled,
+                           const gchar *connectivity_uri,
+                           gint connectivity_interval,
+                           const gchar *connectivity_response,
                            GError **error);
 
 NMManager *nm_manager_get (void);
@@ -99,12 +103,12 @@ NMDevice *nm_manager_get_device_by_master (NMManager *manager,
 					   const char *master,
 					   const char *driver);
 
-const char * nm_manager_activate_connection (NMManager *manager,
-                                             NMConnection *connection,
-                                             const char *specific_object,
-                                             const char *device_path,
-                                             const char *dbus_sender, /* NULL if automatic */
-                                             GError **error);
+NMActiveConnection *nm_manager_activate_connection (NMManager *manager,
+                                                    NMConnection *connection,
+                                                    const char *specific_object,
+                                                    const char *device_path,
+                                                    const char *dbus_sender, /* NULL if automatic */
+                                                    GError **error);
 
 gboolean nm_manager_deactivate_connection (NMManager *manager,
                                            const char *connection_path,
