@@ -55,6 +55,7 @@ struct NMPolicy {
 	gulong vpn_deactivated_id;
 
 	NMFirewallManager *fw_manager;
+	gulong fw_started_id;
 
 	NMSettings *settings;
 
@@ -728,39 +729,6 @@ activate_data_free (ActivateData *data)
 	g_free (data);
 }
 
-static gboolean
-check_master_dependency (NMManager *manager, NMDevice *device, NMConnection *connection)
-{
-	NMSettingConnection *s_con;
-	NMDevice *master_device;
-	const char *master;
-	NMActRequest *req;
-
-	s_con = nm_connection_get_setting_connection (connection);
-	g_assert (s_con);
-
-	master = nm_setting_connection_get_master (s_con);
-
-	/* no master defined, proceed with activation */
-	if (!master)
-		return TRUE;
-
-	master_device = nm_manager_get_device_by_master (manager, master, NULL);
-
-	/* If master device is not yet present, postpone activation until later */
-	if (!master_device)
-		return FALSE;
-
-	/* Make all slaves wait for the master connection to activate. */
-	req = nm_device_get_act_request (master_device);
-	if (!req || !nm_act_request_get_connection (req))
-		return FALSE;
-
-	nm_device_set_master (device, master_device);
-
-	return TRUE;
-}
-
 static gboolean
 auto_activate_device (gpointer user_data)
 {
@@ -817,12 +785,6 @@ auto_activate_device (gpointer user_data)
 	if (best_connection) {
 		GError *error = NULL;
 
-		if (!check_master_dependency (data->policy->manager, data->device, best_connection)) {
-			nm_log_info (LOGD_DEVICE, "Connection '%s' auto-activation postponed: master not available",
-			             nm_connection_get_id (best_connection));
-			goto postpone;
-		}
-
 		nm_log_info (LOGD_DEVICE, "Auto-activating connection '%s'.",
 		             nm_connection_get_id (best_connection));
 		if (!nm_manager_activate_connection (policy->manager,
@@ -839,7 +801,6 @@ auto_activate_device (gpointer user_data)
 		}
 	}
 
- postpone:
 	g_slist_free (connections);
 
  out:
@@ -1290,7 +1251,7 @@ connections_loaded (NMSettings *settings, gpointer user_data)
 }
 
 static void
-add_to_zone_cb (GError *error,
+add_or_change_zone_cb (GError *error,
                 gpointer user_data1,
                 gpointer user_data2)
 {
@@ -1304,23 +1265,53 @@ add_to_zone_cb (GError *error,
 }
 
 static void
-inform_firewall_about_zone (NMPolicy *policy, NMConnection *connection)
+firewall_update_zone (NMPolicy *policy, NMConnection *connection)
 {
 	NMSettingConnection *s_con = nm_connection_get_setting_connection (connection);
 	GSList *iter, *devices;
 
 	devices = nm_manager_get_devices (policy->manager);
+	/* find dev with passed connection and change zone its interface belongs to */
 	for (iter = devices; iter; iter = g_slist_next (iter)) {
 		NMDevice *dev = NM_DEVICE (iter->data);
 
 		if (   (get_device_connection (dev) == connection)
 		    && (nm_device_get_state (dev) == NM_DEVICE_STATE_ACTIVATED)) {
-			nm_firewall_manager_add_to_zone (policy->fw_manager,
-			                                 nm_device_get_ip_iface (dev),
-			                                 nm_setting_connection_get_zone (s_con),
-			                                 add_to_zone_cb,
-			                                 g_object_ref (dev),
-			                                 NULL);
+			nm_firewall_manager_add_or_change_zone (policy->fw_manager,
+			                                        nm_device_get_ip_iface (dev),
+			                                        nm_setting_connection_get_zone (s_con),
+			                                        FALSE, /* change zone */
+			                                        add_or_change_zone_cb,
+			                                        g_object_ref (dev),
+			                                        NULL);
+		}
+	}
+}
+
+static void
+firewall_started (NMFirewallManager *manager,
+                  gpointer user_data)
+{
+	NMPolicy *policy = (NMPolicy *) user_data;
+	NMConnection *connection;
+	NMSettingConnection *s_con;
+	GSList *iter, *devices;
+
+	devices = nm_manager_get_devices (policy->manager);
+	/* add interface of each device to correct zone */
+	for (iter = devices; iter; iter = g_slist_next (iter)) {
+		NMDevice *dev = NM_DEVICE (iter->data);
+
+		connection = get_device_connection (dev);
+		s_con = nm_connection_get_setting_connection (connection);
+		if (nm_device_get_state (dev) == NM_DEVICE_STATE_ACTIVATED) {
+			nm_firewall_manager_add_or_change_zone (policy->fw_manager,
+			                                        nm_device_get_ip_iface (dev),
+			                                        nm_setting_connection_get_zone (s_con),
+			                                        TRUE, /* add zone */
+			                                        add_or_change_zone_cb,
+			                                        g_object_ref (dev),
+			                                        NULL);
 		}
 	}
 }
@@ -1332,7 +1323,7 @@ connection_updated (NMSettings *settings,
 {
 	NMPolicy *policy = (NMPolicy *) user_data;
 
-	inform_firewall_about_zone (policy, connection);
+	firewall_update_zone (policy, connection);
 
 	/* Reset auto retries back to default since connection was updated */
 	set_connection_auto_retries (connection, RETRIES_DEFAULT);
@@ -1456,6 +1447,9 @@ nm_policy_new (NMManager *manager,
 	policy->vpn_deactivated_id = id;
 
 	policy->fw_manager = nm_firewall_manager_get();
+	id = g_signal_connect (policy->fw_manager, "started",
+	                       G_CALLBACK (firewall_started), policy);
+	policy->fw_started_id = id;
 
 	_connect_manager_signal (policy, "state-changed", global_state_changed);
 	_connect_manager_signal (policy, "notify::" NM_MANAGER_HOSTNAME, hostname_changed);
@@ -1501,6 +1495,7 @@ nm_policy_destroy (NMPolicy *policy)
 	g_signal_handler_disconnect (policy->vpn_manager, policy->vpn_deactivated_id);
 	g_object_unref (policy->vpn_manager);
 
+	g_signal_handler_disconnect (policy->fw_manager, policy->fw_started_id);
 	g_object_unref (policy->fw_manager);
 
 	for (iter = policy->manager_ids; iter; iter = g_slist_next (iter))
