@@ -1228,25 +1228,22 @@ set_bond_attr (const char *iface, const char *attr, const char *value)
 	char file[FILENAME_MAX];
 	gboolean ret;
 
-	snprintf (file, sizeof(file), "/sys/class/net/%s/bonding/%s",
-	          iface, attr);
-
+	snprintf (file, sizeof (file), "/sys/class/net/%s/bonding/%s", iface, attr);
 	ret = nm_utils_do_sysctl (file, value);
-	if (!ret)
+	if (!ret) {
 		nm_log_warn (LOGD_HW, "(%s): failed to set bonding attribute "
 		             "'%s' to '%s'", iface, attr, value);
+	}
 
 	return ret;
 }
 
 gboolean
-nm_system_apply_bonding_config (NMSettingBond *s_bond)
+nm_system_apply_bonding_config (const char *iface, NMSettingBond *s_bond)
 {
-	const char *name;
-	guint32 i;
+	const char **opts, **iter;
 
-	name = nm_setting_bond_get_interface_name (s_bond);
-	g_assert (name);
+	g_return_val_if_fail (iface != NULL, FALSE);
 
 	/*
 	 * FIXME:
@@ -1267,13 +1264,28 @@ nm_system_apply_bonding_config (NMSettingBond *s_bond)
 	 * the result is pretty much unforeseeable.
 	 */
 
-	for (i = 0; i < nm_setting_bond_get_num_options (s_bond); i++) {
-		const char *key, *value;
+	/* Set bonding options; if the setting didn't specify a value for the
+	 * option then use the default value to ensure there's no leakage of
+	 * options from any previous connections to this one.
+	 */
+	opts = nm_setting_bond_get_valid_options (s_bond);
+	for (iter = opts; iter && *iter; iter++) {
+		const char *value;
+		gboolean is_default = FALSE;
 
-		if (!nm_setting_bond_get_option (s_bond, i, &key, &value))
-			continue;
+		value = nm_setting_bond_get_option_by_name (s_bond, *iter);
+		if (!value) {
+			value = nm_setting_bond_get_option_default (s_bond, *iter);  /* use the default value */
+			is_default = TRUE;
+		}
 
-		set_bond_attr (name, key, value);
+		nm_log_dbg (LOGD_DEVICE, "(%s): setting bond option '%s' to %s'%s'",
+			        iface,
+			        *iter,
+			        is_default ? "default " : "",
+			        value);
+
+		set_bond_attr (iface, *iter, value);
 	}
 
 	return TRUE;
@@ -1309,7 +1321,7 @@ nm_system_add_bonding_master (const char *iface)
 }
 
 static gboolean
-nm_system_iface_compat_enslave (NMDevice *slave, const char *master_name)
+nm_system_iface_compat_enslave (const char *master_iface, const char *slave_iface)
 {
 	struct ifreq ifr;
 	int fd;
@@ -1323,20 +1335,16 @@ nm_system_iface_compat_enslave (NMDevice *slave, const char *master_name)
 		return FALSE;
 	}
 
-	strncpy (ifr.ifr_name, master_name, IFNAMSIZ);
-	strncpy (ifr.ifr_slave, nm_device_get_iface (slave), IFNAMSIZ);
+	strncpy (ifr.ifr_name, master_iface, IFNAMSIZ);
+	strncpy (ifr.ifr_slave, slave_iface, IFNAMSIZ);
 
 	if (ioctl (fd, SIOCBONDENSLAVE, &ifr) < 0 &&
 	    ioctl (fd, BOND_ENSLAVE_OLD, &ifr) < 0) {
 		nm_log_err (LOGD_DEVICE, "(%s): error enslaving %s: %d (%s)",
-		            master_name, nm_device_get_iface (slave),
-		            errno, strerror (errno));
-		goto errout;
-	}
+		            master_iface, slave_iface, errno, strerror (errno));
+	} else
+		ret = TRUE;
 
-	ret = TRUE;
-
-errout:
 	close (fd);
 
 	return ret;
@@ -1344,58 +1352,56 @@ errout:
 
 /**
  * nm_system_iface_enslave:
- * @slave: Slave device
- * @master: Master device
+ * @master_ifindex: master device interface index
+ * @master_iface: master device interface name
+ * @slave_ifindex: slave device interface index
+ * @slave_iface: slave device interface name
  *
  * Enslaves the 'slave' to 'master. This function targets implementing a
  * generic interface to attaching all kinds of slaves to masters. Currently
  * only bonding is properly supported due to the backwards compatibility
  * function being bonding specific.
  *
- * The slave device needs to be down as a prerequirement.
+ * The slave device needs to be down as a prerequisite.
  *
  * Returns: %TRUE on success, or %FALSE
  */
 gboolean
-nm_system_iface_enslave (NMDevice *slave, NMDevice *master)
+nm_system_iface_enslave (gint master_ifindex,
+                         const char *master_iface,
+                         gint slave_ifindex,
+                         const char *slave_iface)
 {
 	struct nl_sock *sock;
-	const char *master_name;
-	int err, master_ifindex, slave_ifindex;
+	int err;
 
-	master_name = nm_device_get_iface (master);
-	if (!master_name)
-		return FALSE;
+	g_return_val_if_fail (master_ifindex >= 0, FALSE);
+	g_return_val_if_fail (master_iface != NULL, FALSE);
+	g_return_val_if_fail (slave_ifindex >= 0, FALSE);
+	g_return_val_if_fail (slave_iface != NULL, FALSE);
 
 	sock = nm_netlink_get_default_handle ();
 
-	master_ifindex = nm_netlink_iface_to_index (master_name);
-	g_assert (master_ifindex > 0);
-
 	if (!(nm_system_iface_get_flags (master_ifindex) & IFF_MASTER)) {
-		nm_log_err (LOGD_DEVICE, "(%s): interface is not a master", master_name);
+		nm_log_err (LOGD_DEVICE, "(%s): interface is not a master", master_iface);
 		return FALSE;
 	}
 
-	slave_ifindex = nm_device_get_ifindex (slave);
-	g_assert (slave_ifindex > 0);
-
 	g_assert (!nm_system_iface_is_up (slave_ifindex));
 
 	if (nm_system_iface_get_flags (slave_ifindex) & IFF_SLAVE) {
 		nm_log_err (LOGD_DEVICE, "(%s): %s is already a slave",
-		            master_name, nm_device_get_iface (slave));
+		            master_iface, slave_iface);
 		return FALSE;
 	}
 
 	err = rtnl_link_bond_enslave_ifindex (sock, master_ifindex, slave_ifindex);
 	if (err == -NLE_OPNOTSUPP)
-		return nm_system_iface_compat_enslave (slave, master_name);
+		return nm_system_iface_compat_enslave (master_iface, slave_iface);
 
 	if (err < 0) {
 		nm_log_err (LOGD_DEVICE, "(%s): error enslaving %s: %d (%s)",
-		            master_name, nm_device_get_iface (slave),
-		            err, nl_geterror (err));
+		            master_iface, slave_iface, err, nl_geterror (err));
 		return FALSE;
 	}
 
@@ -1403,7 +1409,7 @@ nm_system_iface_enslave (NMDevice *slave, NMDevice *master)
 }
 
 static gboolean
-nm_system_iface_compat_release (NMDevice *device, const char *master_name)
+nm_system_iface_compat_release (const char *master_iface, const char *slave_iface)
 {
 	struct ifreq ifr;
 	int fd;
@@ -1417,29 +1423,26 @@ nm_system_iface_compat_release (NMDevice *device, const char *master_name)
 		return FALSE;
 	}
 
-	strncpy (ifr.ifr_name, master_name, IFNAMSIZ);
-	strncpy (ifr.ifr_slave, nm_device_get_iface (device), IFNAMSIZ);
+	strncpy (ifr.ifr_name, master_iface, IFNAMSIZ);
+	strncpy (ifr.ifr_slave, slave_iface, IFNAMSIZ);
 
 	if (ioctl (fd, SIOCBONDRELEASE, &ifr) < 0 &&
 	    ioctl (fd, BOND_RELEASE_OLD, &ifr) < 0) {
 		nm_log_err (LOGD_DEVICE, "(%s): error releasing slave %s: %d (%s)",
-		            master_name, nm_device_get_iface (device),
-		            errno, strerror (errno));
-		goto errout;
-	}
+		            master_iface, slave_iface, errno, strerror (errno));
+	} else
+		ret = TRUE;
 
-	ret = TRUE;
-
-errout:
 	close (fd);
-
 	return ret;
 }
 
 /**
  * nm_system_iface_release:
- * @slave: Slave device
- * @maser: Master device
+ * @master_ifindex: master device interface index
+ * @master_iface: master device interface name
+ * @slave_ifindex: slave device interface index
+ * @slave_iface: slave device interface name
  *
  * Releases the 'slave' which is attached to 'master. This function targets
  * implementing a generic interface to releasing all kinds of slaves. Currently
@@ -1449,37 +1452,34 @@ errout:
  * Returns: %TRUE on success, or %FALSE
  */
 gboolean
-nm_system_iface_release (NMDevice *slave, NMDevice *master)
+nm_system_iface_release (gint master_ifindex,
+                         const char *master_iface,
+                         gint slave_ifindex,
+                         const char *slave_iface)
 {
 	struct nl_sock *sock;
-	const char *master_name;
-	int err, slave_ifindex;
+	int err;
 
-	master_name = nm_device_get_iface (master);
-	if (!master_name)
-		return TRUE;
+	g_return_val_if_fail (master_ifindex >= 0, FALSE);
+	g_return_val_if_fail (master_iface != NULL, FALSE);
+	g_return_val_if_fail (slave_ifindex >= 0, FALSE);
+	g_return_val_if_fail (slave_iface != NULL, FALSE);
 
 	sock = nm_netlink_get_default_handle ();
 
-	slave_ifindex = nm_device_get_ifindex (slave);
-	g_assert (slave_ifindex > 0);
-
 	/* Only release if this is actually a slave */
 	if (!(nm_system_iface_get_flags (slave_ifindex) & IFF_SLAVE))
-		goto out;
+		return TRUE;
 
 	err = rtnl_link_bond_release_ifindex (sock, slave_ifindex);
 	if (err == -NLE_OPNOTSUPP)
-		return nm_system_iface_compat_release (slave, master_name);
-
-	if (err < 0) {
+		return nm_system_iface_compat_release (master_iface, slave_iface);
+	else if (err < 0) {
 		nm_log_err (LOGD_DEVICE, "(%s): error releasing slave %s: %d (%s)",
-		            master_name, nm_device_get_iface (slave),
-		            err, nl_geterror (err));
+		            master_iface, slave_iface, err, nl_geterror (err));
 		return FALSE;
 	}
 
-out:
 	return TRUE;
 }
 
@@ -1574,11 +1574,221 @@ nm_system_get_iface_vlan_info (int ifindex,
 	return success;
 }
 
+static gboolean
+nm_system_iface_compat_set_name (const char *old_name, const char *new_name)
+{
+	int fd;
+	struct ifreq ifr;
+
+	if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
+		nm_log_err (LOGD_DEVICE, "couldn't open control socket.");
+		return -1;
+	}
+
+	memset (&ifr, 0, sizeof (struct ifreq));
+	strncpy (ifr.ifr_name, old_name, sizeof (ifr.ifr_name));
+	strncpy (ifr.ifr_newname, new_name, sizeof (ifr.ifr_newname));
+
+	if (ioctl (fd, SIOCSIFNAME, &ifr) < 0) {
+		nm_log_err (LOGD_DEVICE, "cann't change %s with %s.", old_name, new_name);
+		close (fd);
+		return FALSE;
+	}
+
+	close (fd);
+	return TRUE;
+}
+
+static gboolean
+nm_system_iface_compat_set_vlan_name_type (int name_type)
+{
+	int fd;
+	struct vlan_ioctl_args if_request;
+
+	if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
+		nm_log_err (LOGD_DEVICE, "couldn't open control socket.");
+		return -1;
+	}
+
+	memset (&if_request, 0, sizeof (struct vlan_ioctl_args));
+	if_request.cmd = SET_VLAN_NAME_TYPE_CMD;
+	if_request.u.name_type = name_type;
+
+	if (ioctl (fd, SIOCSIFVLAN, &if_request) < 0) {
+		nm_log_err (LOGD_DEVICE, "couldn't set name type.");
+		close (fd);
+		return FALSE;
+	}
+
+	close (fd);
+	return TRUE;
+}
+
+static gboolean
+nm_system_iface_compat_add_vlan_device (const char *master, int vid)
+{
+	int fd;
+	struct vlan_ioctl_args if_request;
+
+	g_return_val_if_fail (master, FALSE);
+	g_return_val_if_fail (vid < 4096, FALSE);
+
+	/*
+	 * use VLAN_NAME_TYPE_RAW_PLUS_VID_NO_PAD as default,
+	 * we will overwrite it with rtnl_link_set_name() later.
+	 */
+	if (!nm_system_iface_compat_set_vlan_name_type (VLAN_NAME_TYPE_RAW_PLUS_VID_NO_PAD))
+		return FALSE;
+
+	if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
+		nm_log_err (LOGD_DEVICE, "couldn't open control socket.");
+		return -1;
+	}
+
+	memset (&if_request, 0, sizeof (struct vlan_ioctl_args));
+	strcpy (if_request.device1, master);
+	if_request.cmd = ADD_VLAN_CMD;
+	if_request.u.VID = vid;
+
+	if (ioctl (fd, SIOCSIFVLAN, &if_request) < 0) {
+		nm_log_err (LOGD_DEVICE, "couldn't add vlan device %s vid %d.", master, vid);
+		close (fd);
+		return FALSE;
+	}
+
+	close (fd);
+	return TRUE;
+}
+
+static gboolean
+nm_system_iface_compat_rem_vlan_device (const char *iface)
+{
+	int fd;
+	struct vlan_ioctl_args if_request;
+
+	if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0) {
+		nm_log_err (LOGD_DEVICE, "couldn't open control socket.");
+		return -1;
+	}
+
+	memset (&if_request, 0, sizeof (struct vlan_ioctl_args));
+        strcpy (if_request.device1, iface);
+	if_request.cmd = DEL_VLAN_CMD;
+
+	if (ioctl (fd, SIOCSIFVLAN, &if_request) < 0) {
+		nm_log_err (LOGD_DEVICE, "couldn't rem vlan device %s.", iface);
+		close (fd);
+		return FALSE;
+	}
+
+	close (fd);
+	return TRUE;
+}
+
+static gboolean
+nm_system_iface_compat_add_vlan (NMConnection *connection,
+				const char *iface,
+				int master_ifindex)
+{
+	NMSettingVlan *s_vlan;
+	int vlan_id;
+	guint32 vlan_flags = 0;
+	guint32 num, i, from, to;
+	int ifindex;
+	struct rtnl_link *new_link = NULL;
+	char *master = nm_netlink_index_to_iface (master_ifindex);
+	char *name = NULL;
+
+	s_vlan = nm_connection_get_setting_vlan (connection);
+	g_return_val_if_fail (s_vlan, FALSE);
+
+	vlan_id = nm_setting_vlan_get_id (s_vlan);
+
+	if (!iface) {
+		iface = nm_connection_get_virtual_iface_name (connection);
+		g_return_val_if_fail (iface != NULL, FALSE);
+	}
+
+	/*
+	 * Use VLAN_NAME_TYPE_RAW_PLUS_VID_NO_PAD as default,
+	 * we will overwrite it with rtnl_link_set_name() later.
+	 */
+	name = nm_utils_new_vlan_name(master, vlan_id);
+
+	/*
+	 * vconfig add
+	 */
+
+	if (!nm_system_iface_compat_add_vlan_device (master, vlan_id))
+		goto err_out;
+
+	/*
+	 * get corresponding rtnl_link
+	 */
+
+	if (!nm_system_iface_compat_set_name (name, iface))
+		goto err_out_delete_vlan_with_default_name;
+
+	ifindex = nm_netlink_iface_to_index (iface);
+	if (ifindex <= 0)
+		goto err_out;
+
+	new_link = nm_netlink_index_to_rtnl_link (ifindex);
+	if (!new_link)
+		goto err_out_delete_vlan_with_default_name;
+
+	/*
+	 * vconfig set_flag
+	 */
+	vlan_flags = nm_setting_vlan_get_flags (s_vlan);
+	if (vlan_flags)
+		if (rtnl_link_vlan_set_flags (new_link, vlan_flags))
+			goto err_out_delete_vlan_with_new_name;
+
+	/*
+	 * vconfig set_ingress_map
+	 */
+	num = nm_setting_vlan_get_num_priorities (s_vlan, NM_VLAN_INGRESS_MAP);
+	for (i = 0; i < num; i++) {
+		if (nm_setting_vlan_get_priority (s_vlan, NM_VLAN_INGRESS_MAP, i, &from, &to))
+			if (rtnl_link_vlan_set_ingress_map (new_link, from, to))
+				goto err_out_delete_vlan_with_new_name;
+	}
+
+	/*
+	 * vconfig set_egress_map
+	 */
+	num = nm_setting_vlan_get_num_priorities (s_vlan, NM_VLAN_EGRESS_MAP);
+	for (i = 0; i < num; i++) {
+		if (nm_setting_vlan_get_priority (s_vlan, NM_VLAN_EGRESS_MAP, i, &from, &to))
+			if (rtnl_link_vlan_set_egress_map (new_link, from, to))
+				goto err_out_delete_vlan_with_new_name;
+	}
+
+	rtnl_link_put (new_link);
+	return TRUE;
+
+err_out:
+	g_free (name);
+	return FALSE;
+
+err_out_delete_vlan_with_default_name:
+	nm_system_iface_compat_rem_vlan_device (name);
+	g_free (name);
+	return FALSE;
+
+err_out_delete_vlan_with_new_name:
+	rtnl_link_put (new_link);
+	nm_system_iface_compat_rem_vlan_device (iface);
+	g_free (name);
+	return FALSE;
+}
+
 /**
  * nm_system_add_vlan_iface:
  * @connection: the #NMConnection that describes the VLAN interface
  * @iface: the interface name of the new VLAN interface
- * @master_ifindex: the interface index of the new VLAN interface's master
+ * @parent_ifindex: the interface index of the new VLAN interface's master
  *  interface
  *
  * Add a VLAN device named @iface and specified in @connection.
@@ -1588,7 +1798,7 @@ nm_system_get_iface_vlan_info (int ifindex,
 gboolean
 nm_system_add_vlan_iface (NMConnection *connection,
                           const char *iface,
-                          int master_ifindex)
+                          int parent_ifindex)
 {
 	NMSettingVlan *s_vlan;
 	int ret = -1;
@@ -1598,7 +1808,7 @@ nm_system_add_vlan_iface (NMConnection *connection,
 	guint32 vlan_flags = 0;
 	guint32 num, i, from, to;
 
-	g_return_val_if_fail (master_ifindex >= 0, FALSE);
+	g_return_val_if_fail (parent_ifindex >= 0, FALSE);
 
 	nlh = nm_netlink_get_default_handle ();
 	g_return_val_if_fail (nlh != NULL, FALSE);
@@ -1620,10 +1830,17 @@ nm_system_add_vlan_iface (NMConnection *connection,
 	}
 
 	ret = rtnl_link_set_type (new_link, "vlan");
-	if (ret < 0)
+	if (ret == -NLE_OPNOTSUPP) {
+		/*
+		 * There is no linbl3, try ioctl.
+		 */
+		ret = -1;
+		if (nm_system_iface_compat_add_vlan (connection, iface, parent_ifindex))
+			ret = 0;
 		goto out;
+	}
 
-	rtnl_link_set_link (new_link, master_ifindex);
+	rtnl_link_set_link (new_link, parent_ifindex);
 	rtnl_link_set_name (new_link, iface);
 	rtnl_link_vlan_set_id (new_link, vlan_id);
 
@@ -1693,9 +1910,17 @@ nm_system_del_vlan_iface (const char *iface)
 	new_link = rtnl_link_get_by_name (cache, iface);
 	if (new_link) {
 		ret = rtnl_link_delete (nlh, new_link);
-		rtnl_link_put (new_link);
+		if (ret == -NLE_OPNOTSUPP) {
+			/*
+			 * There is no linbl3, try ioctl.
+			 */
+			ret = -1;
+			if (nm_system_iface_compat_rem_vlan_device (iface))
+				ret = 0;
+		}
 	}
 
+	rtnl_link_put (new_link);
 	nl_cache_free (cache);
 	return (ret == 0) ? TRUE : FALSE;
 }
